<?php

use PrestaShop\PrestaShop\Core\Payment\PaymentOption;

if (!defined('_PS_VERSION_')) {
    exit;
}

class Tucompra extends PaymentModule{

    private $_html = '';
    public $id_commerce;
    public $key_encryption;
    public $is_demo;
    public $gateway_demo;
    public $gateway_production;

    public function __construct(){
        $this->name = 'tucompra';
        $this->tab = 'payments_gateways';
        $this->version = '3.1';
        $this->ps_versions_compliancy = array('min' => '1.7', 'max' => _PS_VERSION_);
        $this->author = 'PrestaShop';
        $this->controllers = array('validation');
        $this->is_eu_compatible = 1;

        $this->currencies = true;
        $this->currencies_mode = 'checkbox';
        $this->bootstrap = true;

		$this->_errors = array();
        $this->warning = array();

		$config = Configuration::getMultiple(array('TC_ID_COMMERCE', 'TC_KEY_ENCRYPTION', 'TC_IS_DEMO', 'TC_GATEWAY_DEMO', 'TC_GATEWAY_PRODUCTION'));

        if (isset($config['TC_ID_COMMERCE']))
            $this->id_commerce = $config['TC_ID_COMMERCE'];
        if (isset($config['TC_KEY_ENCRYPTION']))
            $this->key_encryption = $config['TC_KEY_ENCRYPTION'];
        if (isset($config['TC_IS_DEMO']))
            $this->is_demo = $config['TC_IS_DEMO'];
        if (isset($config['TC_GATEWAY_DEMO']))
            $this->gateway_demo = $config['TC_GATEWAY_DEMO'];
        if (isset($config['TC_GATEWAY_PRODUCTION']))
            $this->gateway_production = $config['TC_GATEWAY_PRODUCTION'];

        parent::__construct();

        $this->displayName = $this->l('tucompra');
        $this->description = $this->l('Description of Payment Example');

        if (!count(Currency::checkPaymentCurrencies($this->id))) {
            $this->warning = $this->l('No currency has been set for this module.');
        }
    }

    public function install(){
        $languages = Language::getLanguages(false);

        $name_state = array();

        foreach ($languages AS $language) {
            $name_state[$language['id_lang']] = 'Pending confirmation TuCompra';
        }

        $OrderState = new OrderState();
        $OrderState->name = $name_state;
        $OrderState->send_email = 0;
        $OrderState->invoice = 0;
        $OrderState->logable = false;
        $OrderState->color = '#9C9C9C';
        if ($OrderState->add()) {
            Configuration::updateValue('PS_OS_TUCOMPRA', $OrderState->id);
            copy(_PS_MODULE_DIR_ . $this->name . '/logo.gif', _PS_IMG_DIR_ . 'os/' . $OrderState->id . '.gif');
        }
        else
            return false;

        if (!parent::install() OR
                !$this->registerHook('payment') OR
		!$this->registerHook('paymentOptions') OR
                !$this->registerHook('paymentReturn') OR
                !Configuration::updateValue('TC_ID_COMMERCE', '') OR
                !Configuration::updateValue('TC_KEY_ENCRYPTION', '') OR
                !Configuration::updateValue('TC_IS_DEMO', '1') OR
                !Configuration::updateValue('TC_GATEWAY_DEMO', 'https://demover3-1.tucompra.net/tc/app/inputs/compra.jsp') OR
                !Configuration::updateValue('TC_GATEWAY_PRODUCTION', 'https://gateway2.tucompra.com.co/tc/app/inputs/compra.jsp'))
            return false;
        return true;
    }

    public function uninstall() {
        if (parent::uninstall()) {
            Configuration::deleteByName('PS_OS_TUCOMPRA');
            Configuration::deleteByName('TC_ID_COMMERCE');
            Configuration::deleteByName('TC_KEY_ENCRYPTION');
            Configuration::deleteByName('TC_IS_DEMO');
            Configuration::deleteByName('TC_GATEWAY_DEMO');
            Configuration::deleteByName('TC_GATEWAY_PRODUCTION');

            return true;
        }
        return false;
    }

	    public function getContent() {
        $this->_html = '<img src="' . $this->_path . 'img/tucompra.png' . '"/><h2>' . $this->displayName . ' - v' . $this->version . '</h2>';

        if (Tools::isSubmit('submitSettings')) {
            Configuration::updateValue('TC_ID_COMMERCE', Tools::getValue('id_commerce'));
            Configuration::updateValue('TC_KEY_ENCRYPTION', Tools::getValue('key_encryption'));
            Configuration::updateValue('TC_IS_DEMO', Tools::getValue('is_demo'));
            Configuration::updateValue('TC_GATEWAY_DEMO', Tools::getValue('gateway_demo'));
            Configuration::updateValue('TC_GATEWAY_PRODUCTION', Tools::getValue('gateway_production'));

            $this->__construct();

            if (!sizeof($this->_errors))
                $this->_html .= $this->displayConfirmation($this->l('Settings updated'));
        }

        $this->displayErrors();
        $this->_displayForm();

        return $this->_html;
    }

    private function _displayForm() {
        global $smarty, $cookie;

        $paramsBack = array('TUCOMPRA_DIR' => $this->_path,
            'TUCOMPRA_DIR_FULL' => floatval(substr(_PS_VERSION_, 0, 3)) < 1.4 ? ((Configuration::get('PS_SSL_ENABLED') ? 'https://' : 'http://') . htmlspecialchars($_SERVER['HTTP_HOST'], ENT_COMPAT, 'UTF-8') . __PS_BASE_URI__ . 'modules/' . $this->name . '/') : (Tools::getShopDomainSsl(true, true) . __PS_BASE_URI__ . 'modules/' . $this->name . '/'),
            'TUCOMPRA_IMG' => $this->_path . 'img/',
            'ID_COMMERCE' => $this->id_commerce,
            'KEY_ENCRYPTION' => $this->key_encryption,
            'IS_DEMO' => $this->is_demo,
            'GATEWAY_DEMO' => $this->gateway_demo,
            'GATEWAY_PRODUCTION' => $this->gateway_production,
            'ACTION_URL' => Tools::safeOutput($_SERVER['PHP_SELF']) . '?' . $_SERVER['QUERY_STRING']);

        if(version_compare(_PS_VERSION_, '1.5') >= 0) {
            $link = new Link();
            $paramsBack['IS_15'] = TRUE;
            $paramsBack['FINISH_URL'] = $link->getModuleLink($this->name, 'finish');
            $paramsBack['CONFIRMATION_URL'] = $link->getModuleLink($this->name, 'confirmation');
        }

        $smarty->assign('paramsBack', $paramsBack);
        $this->_html .= $this->display(__FILE__, 'views/templates/back/config.tpl');

    }

    public function hookPayment($params) {
        if (!$this->active)
            return;

        global $smarty;

        // Check if cart has product download
        foreach ($params['cart']->getProducts() AS $product) {
            $pd = ProductDownload::getIdFromIdProduct((int) ($product['id_product']));
            if ($pd AND Validate::isUnsignedInt($pd))
                return false;
        }

        $smarty->assign(array(
            'this_path' => $this->_path,
            'this_path_ssl' => floatval(substr(_PS_VERSION_, 0, 3)) < 1.4 ? ((Configuration::get('PS_SSL_ENABLED') ? 'https://' : 'http://') . htmlspecialchars($_SERVER['HTTP_HOST'], ENT_COMPAT, 'UTF-8') . __PS_BASE_URI__ . 'modules/' . $this->name . '/') : (Tools::getShopDomainSsl(true, true) . __PS_BASE_URI__ . 'modules/' . $this->name . '/')
        ));

        return $this->display(__FILE__, 'payment.tpl');
    }

    public function hookPaymentReturn($params) {
        if (!$this->active)
            return;

        global $smarty, $cookie;

	$order = $params['order'];
        $state = $order->current_state;

        $customer = new Customer($order->id_customer);
        $firstname = $customer->firstname;
        $lastname = $customer->lastname;
        $email = $customer->email;
        $address = new Address($order->id_address_invoice);
        $country_name = Country::getNameById($cookie->id_lang, $address->id_country);
		$currency = new Currency($order->id_currency);

        if ($state == Configuration::get('PS_OS_TUCOMPRA') OR $state == Configuration::get('PS_OS_OUTOFSTOCK')) {
            $paramsTC = array('ID_COMMERCE' => $this->id_commerce,
                'KEY_ENCRYPTION' => $this->key_encryption,
                'IS_DEMO' => $this->is_demo,
                'GATEWAY_DEMO' => $this->gateway_demo,
                'ID_COMMERCE' => $this->id_commerce,
                'GATEWAY_PRODUCTION' => $this->gateway_production);

			$is_16 = true;
			if (version_compare(_PS_VERSION_, '1.7') < 0)
				$is_16 = false;
            $smarty->assign(array(
				'is_16' => $is_16,
                'status' => true,
                'total_to_pay' => $order->getOrdersTotalPaid(),
                'total' => $order->getOrdersTotalPaid(),
                'order' => $order,
                'currency' => $currency,
                'firstname' => $firstname,
                'lastname' => $lastname,
                'email' => $email,
//                'customer' => $customer,
                'address' => $address,
                'country_name' => $country_name,
                'paramsTC' => $paramsTC
            ));
            $smarty->assign('firstname', $firstname);
        }
        else
            $smarty->assign('status', false);

		$smarty->assign('shop_name', Configuration::get('PS_SHOP_NAME'));
		$smarty->assign('base_dir_ssl', floatval(substr(_PS_VERSION_, 0, 3)) < 1.4 ? ((Configuration::get('PS_SSL_ENABLED') ? 'https://' : 'http://') . htmlspecialchars($_SERVER['HTTP_HOST'], ENT_COMPAT, 'UTF-8') . __PS_BASE_URI__ . 'modules/' . $this->name . '/') : (Tools::getShopDomainSsl(true, true) . __PS_BASE_URI__ . 'modules/' . $this->name . '/'));

        return $this->display(__FILE__, 'payment_return.tpl');
    }

    public function execPayment($cart) {
        if (!$this->active)
            return;

        global $smarty;

        $smarty->assign(array(
            'total' => $cart->getOrderTotal(true, 3),
            'this_path' => $this->_path,
            'this_path_ssl' => floatval(substr(_PS_VERSION_, 0, 3)) < 1.4 ? ((Configuration::get('PS_SSL_ENABLED') ? 'https://' : 'http://') . htmlspecialchars($_SERVER['HTTP_HOST'], ENT_COMPAT, 'UTF-8') . __PS_BASE_URI__ . 'modules/' . $this->name . '/') : (Tools::getShopDomainSsl(true, true) . __PS_BASE_URI__ . 'modules/' . $this->name . '/')
        ));

        return $this->display(__FILE__, 'payment_execution.tpl');
    }

    private function displayErrors($return = true) {
        if (sizeof($this->_errors)) {
            $_html = '';
            $nbErrors = sizeof($this->_errors);

            $_html .= '
    		<div class="alert error">
    			<h3>' . ($nbErrors > 1 ? $this->l('There are') : $this->l('There is')) . ' ' . $nbErrors . ' ' . ($nbErrors > 1 ? $this->l('errors') : $this->l('error')) . '</h3>
    			<ol>';
            foreach ($this->_errors AS $error)
                $_html .= '<li>' . $error . '</li>';
            $_html .= '
    			</ol>
    		</div>';

            if ($return)
                $this->_html = $_html;
            else
                echo $_html;
        }
    }

    public function hookPaymentOptions($params){

        if (!$this->active) {
            return;
        }

        if (!$this->checkCurrency($params['cart'])) {
            return;
        }

		$externalOption = new PaymentOption();
		echo "<script>console.log('Crear PaymentOption');</script>";
        $externalOption->setModuleName('tucompra')
                       ->setLogo(Media::getMediaPath(_PS_MODULE_DIR_.$this->name.'/payment.png'))
					   ->setAction($this->context->link->getModuleLink('tucompra', 'validation', [], true))
					   ->setCallToActionText('Pay by Tucompra');
                       //->setAdditionalInformation($this->context->smarty->fetch('module:tucompra/payment_execution.tpl'))
		echo "<script>console.log('Finalizo a hookPaymentOptions');</script>";

        return [$externalOption];
    }

    public function checkCurrency($cart){
        $currency_order = new Currency($cart->id_currency);
        $currencies_module = $this->getCurrency($cart->id_currency);

        if (is_array($currencies_module)) {
            foreach ($currencies_module as $currency_module) {
                if ($currency_order->id == $currency_module['id_currency']) {
                    return true;
                }
            }
        }
        return false;
    }

}
